from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class OnlineUpdate(ApiGroup):
    def check_new_version(self) -> dict:
        return self._connection.get('online-update/check-new-version')

    @authorized_call
    def set_check_new_version(self) -> dict:
        return self._connection.post('online-update/check-new-version')

    def status(self) -> dict:
        return self._connection.get('online-update/status')

    def url_list(self) -> dict:
        return self._connection.get('online-update/url-list')

    def ack_newversion(self) -> dict:
        return self._connection.get('online-update/ack-newversion')

    def set_ack_newversion(self):
        return self._connection.post('online-update/ack-newversion', {
            'userAckNewVersion': 0
        })

    def cancel_downloading(self) -> dict:
        return self._connection.get('online-update/cancel-downloading')

    def set_cancel_downloading(self):
        return self._connection.post('online-update/cancel-downloading')

    def upgrade_messagebox(self) -> dict:
        return self._connection.get('online-update/upgrade-messagebox')

    def set_upgrade_messagebox(self, messagebox):
        return self._connection.post('online-update/upgrade-messagebox', {
            'messagebox': messagebox
        })




from huawei_lte_api.Connection import Connection
from huawei_lte_api.api.Device import Device
from huawei_lte_api.api.User import User
from huawei_lte_api.api.Monitoring import Monitoring
from huawei_lte_api.api.Security import Security
from huawei_lte_api.api.WebServer import WebServer
from huawei_lte_api.api.WLan import WLan
from huawei_lte_api.api.Cradle import Cradle
from huawei_lte_api.api.Pin import Pin
from huawei_lte_api.config.Global import Global
from huawei_lte_api.config.Voice import Voice
from huawei_lte_api.config.PcAssistant import PcAssistant
from huawei_lte_api.api.Ota import Ota
from huawei_lte_api.api.Net import Net
from huawei_lte_api.api.DialUp import DialUp
from huawei_lte_api.api.Sms import Sms
from huawei_lte_api.api.Redirection import Redirection
from huawei_lte_api.api.VSim import VSim
from huawei_lte_api.api.FileManager import FileManager
from huawei_lte_api.api.Dhcp import Dhcp
from huawei_lte_api.api.DDns import DDns
from huawei_lte_api.api.Diagnosis import Diagnosis
from huawei_lte_api.api.SNtp import SNtp
from huawei_lte_api.api.OnlineUpdate import OnlineUpdate


class Client(object):
    def __init__(self, connection: Connection):
        self.monitoring = Monitoring(connection)
        self.security = Security(connection)
        self.webserver = WebServer(connection)
        self.global_ = Global(connection)
        self.wlan = WLan(connection)
        self.cradle = Cradle(connection)
        self.pin = Pin(connection)
        self.config_global = Global(connection)
        self.config_voice = Voice(connection)
        self.config_pc_assistant = PcAssistant(connection)
        self.ota = Ota(connection)
        self.net = Net(connection)
        self.dial_up = DialUp(connection)
        self.sms = Sms(connection)
        self.redirection = Redirection(connection)
        self.v_sim = VSim(connection)
        self.file_manager = FileManager(connection)
        self.dhcp = Dhcp(connection)
        self.d_dns = DDns(connection)
        self.diagnosis = Diagnosis(connection)
        self.s_ntp = SNtp(connection)
        self.user = User(connection)
        self.device = Device(connection)
        self.online_update = OnlineUpdate(connection)



from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class DDns(ApiGroup):
    @authorized_call
    def get_ddns_list(self) -> dict:
        return self._connection.get('ddns/ddns-list')

    def get_status(self) ->dict:
        return self._connection.get('ddns/status')


import datetime
from urllib.parse import urlparse
from huawei_lte_api.Connection import Connection
from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.api.User import User


def authorized_call(fn):
    def wrapped(*args, **kw):
        # First argument should be self of class
        if not isinstance(args[0], ApiGroup):
            raise Exception('{} is not instance of ApiGroup'.format(args[0]))

        if not isinstance(args[0]._connection, AuthorizedConnection):
            raise Exception('{} is not instance of AuthorizedConnection needed by {}'.format(args[0]._connection, fn))

        if not args[0]._connection.enforce_authorized_connection():
            raise Exception('{}: Failed to enforce login for  {}'.format(args[0]._connection, fn))

        return fn(*args, **kw)
    return wrapped


class AuthorizedConnection(Connection):
    LOGOUT_TIMEOUT = 300  # seconds
    login_time = None
    logged_in = False

    def __init__(self, url: str, username: str=None, password: str=None, login_on_demand: bool=False):
        super(AuthorizedConnection, self).__init__(url)
        parsed_url = urlparse(url)

        username = username if username else parsed_url.username
        password = password if password else parsed_url.password

        self.user = User(self, username, password)

        if not login_on_demand:
            if self.user.login(True):
                self.login_time = datetime.datetime.utcnow()
                self.logged_in = True

    def _is_login_timeout(self) -> bool:
        if self.login_time is None:
            return True
        logout_time = self.login_time + datetime.timedelta(seconds=self.LOGOUT_TIMEOUT)
        return logout_time < datetime.datetime.utcnow()

    def enforce_authorized_connection(self):
        # Check if connection timeouted or not
        if not self.logged_in or self._is_login_timeout():
            # Connection timeouted, relogin
            if self.user.login():
                self.login_time = datetime.datetime.utcnow()
                self.logged_in = True
            else:
                self.login_time = None
                self.logged_in = False

        return self.logged_in





from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class DialUp(ApiGroup):
    def mobile_dataswitch(self) ->dict:
        return self._connection.get('dialup/mobile-dataswitch')

    def connection(self) ->dict:
        return self._connection.get('dialup/connection')

    def dialup_feature_switch(self) ->dict:
        return self._connection.get('dialup/dialup-feature-switch')

    @authorized_call
    def profiles(self) ->dict:
        return self._connection.get('dialup/profiles')

    def auto_apn(self) ->dict:
        return self._connection.get('dialup/auto-apn')

    def dial(self):
        return self._connection.post('dialup/dial', {
            'Action': 1
        })

    def set_mobile_dataswitch(self, dataswitch: int=0):
        return self._connection.post('dialup/mobile-dataswitch', {
            'dataswitch': dataswitch
        })


class ResponseErrorException(Exception):
    def __init__(self, message, code):
        super(ResponseErrorException, self).__init__(message)
        self.code = code


class ResponseErrorNotSupportedException(ResponseErrorException):
    pass


class ResponseErrorLoginRequiredException(ResponseErrorException):
    pass


class ResponseErrorSystemBusyException(ResponseErrorException):
    pass


class ResponseErrorLoginCsfrException(ResponseErrorException):
    pass


class LoginErrorUsernameWrongException(ResponseErrorException):
    pass


class LoginErrorPasswordWrongException(ResponseErrorException):
    pass


class LoginErrorAlreadyLoginException(ResponseErrorException):
    pass


class LoginErrorUsernamePasswordWrongException(ResponseErrorException):
    pass


class LoginErrorUsernamePasswordOverrunException(ResponseErrorException):
    pass


class LoginErrorUsernamePasswordModifyException(ResponseErrorException):
    pass




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Language(ApiGroup):
    def set_current_language(self, current_language: str):
        return self._connection.post('language/current-language', {
            'CurrentLanguage': current_language
        })




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Diagnosis(ApiGroup):
    def trace_route_result(self) -> dict:
        return self._connection.get('diagnosis/tracerouteresult')



import sys
from setuptools import setup

if sys.version_info >= (3, 0):
    long_description = open('README.md', encoding='utf-8').read()
else:
    long_description = open('README.md').read()

setup(
    name='huawei-lte-api',
    version='1.0.12',
    packages=[
        'huawei_lte_api',
        'huawei_lte_api.api',
        'huawei_lte_api.enums',
        'huawei_lte_api.config'
    ],
    install_requires=[
        'requests',
        'dicttoxml',
        'xmltodict',
    ],
    url='https://github.com/Salamek/huawei-lte-api',
    license='LGPL-3.0 ',
    author='Adam Schubert',
    author_email='adam.schubert@sg1-game.net',
    description='API For huawei LAN/WAN LTE Modems',
    long_description=long_description,
    long_description_content_type='text/markdown',
    test_suite='tests',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.5',
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development',
    ],
    setup_requires=[
        'pytest-runner'
    ],
    tests_require=[
        'pytest',
        'pylint',
        'pytest-cov'
    ]

)




from huawei_lte_api.ApiGroup import ApiGroup
from huawei_lte_api.AuthorizedConnection import authorized_call


class Redirection(ApiGroup):
    def homepage(self) -> dict:
        return self._connection.get('redirection/homepage')